/*
Program  mcrlf2  converts text files from DOS to UNIX etc.
Coded by Petr Mikulik, http://www.sci.muni.cz/~mikulik/, 19. 6. 1998

Freeware version.

Compilation:
OS/2: gcc -O2 -s -o mcrlf2.exe mcrlf2.c
Unix: gcc -O2 -s -o mcrlf2 mcrlf2.c

Note 1: this program could work also as a filter. However, this
option is taken away because some systems automatically expand LF
going to stdout to CRLF.
Note 2: opening temporary file by tmpname() or tempname() failed on
FreeBSD, thus always use file called "$mcrlf2$.$$$".
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <utime.h>

#define LF 0x0A  /* '\n' */
#define CR 0x0D  /* '\r' */

#define CRLFtoLF 0
#define LFtoCRLF 1
#define CRtoLF   2
#define CRtoCRLF 3
#define LFtoCR   4
#define DeleteCR 5
#define DeleteLF 6


void Usage ( void ) {
fprintf(stderr,"Program  mcrlf2  converts text files from DOS to UNIX etc.\n");
fprintf(stderr,"Coded by Petr Mikulik, http://www.sci.muni.cz/~mikulik/, 19. 6. 1998\n");
fprintf(stderr,"Usage:\n   mcrlf2 option file(s)\n");
fprintf(stderr,"Options:\n");
fprintf(stderr,"   -u ... Unix (LF) to DOS (CRLF)\n");
fprintf(stderr,"   -d ... DOS (CRLF) to Unix (LF)\n");
fprintf(stderr,"   -m ... Macintosh (CR) to Unix (LF)\n");
fprintf(stderr,"   -M ... Macintosh (CR) to DOS (CRLF)\n");
fprintf(stderr,"   -U ... Unix (CR) to Macintosh (LF)\n");
fprintf(stderr,"   -c ... delete all CR\n");
fprintf(stderr,"   -l ... delete all LF\n");
fprintf(stderr,"Note: line feed LF=0x0A=10='\\n', carriage return CR=0x0D=13='\\r'\n");
exit(1);
}


void main ( int argc, char **argv ) {
  FILE *in, *out;
  signed char c;
  char *file_name;
  int optind;
  char mode=255;
  /* char *tmpName; */
  char tmpName[] = "$mcrlf2$.$$$";
  struct stat in_stat;
  struct utimbuf outtimes;

#ifdef __EMX__  /* expand wildcards of files passed on the command line */
_wildcard (&argc, &argv);
#endif

if (argc<3) Usage(); /* even though */
/* if (argc<2) Usage(); // this would be in case of OK filtering of CRLFtoLFs */
if (!strcmp(argv[1],"-u")) mode=LFtoCRLF; else
if (!strcmp(argv[1],"-d")) mode=CRLFtoLF; else
if (!strcmp(argv[1],"-c")) mode=DeleteCR; else
if (!strcmp(argv[1],"-l")) mode=DeleteLF; else
if (!strcmp(argv[1],"-m")) mode=CRtoLF; else
if (!strcmp(argv[1],"-M")) mode=CRtoCRLF; else
if (!strcmp(argv[1],"-U")) mode=LFtoCR; else
  Usage();

optind=2;

do {

if (argc==2) {
    /* fprintf(stderr,"Processing stdin in mode %i\n",(int)mode); */
    in=stdin;
    out=stdout; }
  else {
    file_name=argv[optind];
    /* fprintf(stderr,"Processing %s in mode %i\n",file_name,(int)mode); */
    if ( (in = fopen(file_name,"rb")) == NULL) {
      perror(file_name);
      goto next_file;
      }
    /* out=fopen( tmpName=tmpnam(NULL), "wb" ); // open temporary file */
    out=fopen( tmpName, "wb");
    fstat( fileno(in), &in_stat ); /* get timestamp */
    }

while ( (c=fgetc(in)) != EOF ) {
  next:;
  if (c!=CR && c!=LF) { fputc(c,out); continue; }
  if (mode==CRLFtoLF && c==CR) {
    if ( (c=fgetc(in))==EOF ) { fputc(CR,out); break; }
    if (c!=LF) { fputc(CR,out); goto next; }
    fputc(LF,out);
    continue;
    }
  if (mode==LFtoCRLF && c==LF) {
    fputc(CR,out); fputc(LF,out);
    continue;
    }
  if ( (mode==DeleteLF && c==LF) || (mode==DeleteCR && c==CR) )
    continue;
  if (mode==CRtoLF && c==CR) {
    fputc(LF,out);
    continue;
    }
  if (mode==CRtoCRLF && c==CR) {
    fputc(CR,out); fputc(LF,out);
    continue;
    }
  if (mode==LFtoCR && c==LF) {
    fputc(CR,out);
    continue;
    }
  fputc(c,out); /* this should not happen */
  }

if (argc>2) { /* no pipe, thus close and rename files */
  fclose(out); fclose(in);
  remove(file_name);
  rename(tmpName,file_name);
  outtimes.modtime = outtimes.actime = in_stat.st_mtime; /* set filestamp */
  utime(file_name,&outtimes);
  }

next_file:;

optind++;
} /* for each file on the command line or once for filter-like behaviour */
while ( optind<argc );

}

/* eof mcrlf2.c */
